#!/usr/bin/env bash

# Original package: https://github.com/timsutton/osx-vm-templates

# This script will use the setup in this package to create a virtual image for
# macOS following the specifications of the provided version number.
# The build happens in 3 stages:
#
# 1. An installable DMG is created from the original Apple installation app
# 2. A basic OVF image containing the installed ISO is generated
# 3. The final OVA image is created by provisioning the OVF image
#
# It is done in 3 stages so that provisioning can be tested in a faster test
# loop given points 1 and 2 can be very long.

if [[ "$#" == "0" ]]; then
	echo "usage: $0 <macos-version>"
	echo "example: $0 10.9"
	exit 1
fi

# Edit this part and modify it as you see fit
username=gitlab
password=gitlab
curdir=`pwd`
dmgdir="${curdir}/dmg"
ovfdir="${curdir}/ovf"
ovadir="${curdir}/ova"

# Expected app and image names (cannot really guess - wait for outputs and set
# as appropriate)
if [[ "$1" == "10.9" ]]; then
	app="/Applications/Install OS X Mavericks.app"
	dmg="${dmgdir}/OSX_InstallESD_10.9.5_13F34.dmg"
else
	echo "Version $1 is currently unsupported"
	exit 1
fi

short_ver=${1/./}
xcode_sdk_version=${1}
machine_name="macos${short_ver}"
guest_os_type="MacOS${short_ver}_64"
ovf="${ovfdir}/${machine_name}.ovf"
ova="${ovadir}/${machine_name}.ova"

echo "Building virtualbox machine for macOS $1..."
echo "username  = ${username}"
echo "password  = ${password}"
echo "app       = ${app}"
echo "os type   = ${guest_os_type}"
echo "name      = ${machine_name}"
echo "xcode SDK = ${xcode_sdk_version}"
echo "dmg       = ${dmg}"
echo "ovf       = ${ovf}"
echo "ova       = ${ova}"

if [ ! -r "${dmg}" ]; then
  echo "Stage 1: [$(basename ${app})] -> [$(basename ${dmg})]"
	sudo prepare_iso/prepare_iso.sh \
		-u ${username} \
		-p ${password} \
		-i prepare_iso/support/gitlab.jpg \
		-D DISABLE_REMOTE_MANAGEMENT \
		"${app}" "${dmgdir}"
else
  echo "Skipping stage 1: [$(basename ${dmg})] exists. Remove to force rebuild."
fi

if [ ! -r "${ovf}" ]; then
  echo "Stage 2: [$(basename ${dmg})] -> [$(basename ${ovf})]"
  packer build \
    -on-error=ask \
    -only virtualbox-iso \
    -var dmg_url="${dmg}" \
    -var guest_os_type="${guest_os_type}" \
    -var output_directory="${ovfdir}" \
    -var machine_name="${machine_name}" \
    -var username="${username}" \
    -var password="${password}" \
    packer/ovf.json
  if [ -d packer_cache ]; then
    rmdir packer_cache #normally empty
  fi
else
  echo "Skipping stage 2: [$(basename ${ovf})] exists. Remove to force rebuild."
fi

if [ ! -r "${ova}" ]; then
  echo "Stage 3: [$(basename ${ovf})] -> provision -> [$(basename ${ova})]"
  packer build \
    -on-error=ask \
    -only virtualbox-ovf \
    -var source_path="${ovf}" \
    -var guest_os_type="${guest_os_type}" \
    -var output_directory="${ovadir}" \
    -var machine_name="${machine_name}" \
    -var xcode_sdk_version="${xcode_sdk_version}" \
    -var username="${username}" \
    -var password="${password}" \
    -var provisioning_delay=30 \
    packer/provision.json
  if [ -d packer_cache ]; then
    rmdir packer_cache #normally empty
  fi
else
  echo "Skipping stage 3: [$(basename ${ova})] exists. Remove to force rebuild."
fi

echo "Note: To be able to SSH into the host, since it is configure for NAT, follow"
echo "instructions at: https://forums.virtualbox.org/viewtopic.php?f=8&t=55766"
